@@ -2,7 +2,7 @@
 package BerkeleyDB;
 
 
-#     Copyright (c) 1997-2010 Paul Marquess. All rights reserved.
+#     Copyright (c) 1997-2011 Paul Marquess. All rights reserved.
 #     This program is free software; you can redistribute it and/or
 #     modify it under the same terms as Perl itself.
 #
@@ -17,7 +17,7 @@ use Carp;
 use vars qw($VERSION @ISA @EXPORT $AUTOLOAD
 		$use_XSLoader);
 
-$VERSION = '0.42';
+$VERSION = '0.48';
 
 require Exporter;
 #require DynaLoader;
@@ -51,9 +51,11 @@ BEGIN {
 	DB_ARCH_DATA
 	DB_ARCH_LOG
 	DB_ARCH_REMOVE
+	DB_ASSOC_CREATE
 	DB_ASSOC_IMMUTABLE_KEY
 	DB_AUTO_COMMIT
 	DB_BEFORE
+	DB_BOOTSTRAP_HELPER
 	DB_BTREE
 	DB_BTREEMAGIC
 	DB_BTREEOLDVER
@@ -77,6 +79,8 @@ BEGIN {
 	DB_CURSOR_BULK
 	DB_CURSOR_TRANSIENT
 	DB_CXX_NO_EXCEPTIONS
+	DB_DATABASE_LOCK
+	DB_DATABASE_LOCKING
 	DB_DEGREE_2
 	DB_DELETED
 	DB_DELIMITER
@@ -93,6 +97,7 @@ BEGIN {
 	DB_DURABLE_UNKNOWN
 	DB_EID_BROADCAST
 	DB_EID_INVALID
+	DB_EID_MASTER
 	DB_ENCRYPT
 	DB_ENCRYPT_AES
 	DB_ENV_APPINIT
@@ -100,6 +105,7 @@ BEGIN {
 	DB_ENV_CDB
 	DB_ENV_CDB_ALLDB
 	DB_ENV_CREATE
+	DB_ENV_DATABASE_LOCKING
 	DB_ENV_DBLOCAL
 	DB_ENV_DIRECT_DB
 	DB_ENV_DIRECT_LOG
@@ -107,19 +113,20 @@ BEGIN {
 	DB_ENV_DSYNC_LOG
 	DB_ENV_FAILCHK
 	DB_ENV_FATAL
+	DB_ENV_HOTBACKUP
 	DB_ENV_LOCKDOWN
 	DB_ENV_LOCKING
 	DB_ENV_LOGGING
 	DB_ENV_LOG_AUTOREMOVE
 	DB_ENV_LOG_INMEMORY
 	DB_ENV_MULTIVERSION
+	DB_ENV_NOFLUSH
 	DB_ENV_NOLOCKING
 	DB_ENV_NOMMAP
 	DB_ENV_NOPANIC
 	DB_ENV_NO_OUTPUT_SET
 	DB_ENV_OPEN_CALLED
 	DB_ENV_OVERWRITE
-	DB_ENV_PANIC_OK
 	DB_ENV_PRIVATE
 	DB_ENV_RECOVER_FATAL
 	DB_ENV_REF_COUNTED
@@ -147,15 +154,27 @@ BEGIN {
 	DB_EVENT_REG_ALIVE
 	DB_EVENT_REG_PANIC
 	DB_EVENT_REP_CLIENT
+	DB_EVENT_REP_CONNECT_BROKEN
+	DB_EVENT_REP_CONNECT_ESTD
+	DB_EVENT_REP_CONNECT_TRY_FAILED
+	DB_EVENT_REP_DUPMASTER
 	DB_EVENT_REP_ELECTED
+	DB_EVENT_REP_ELECTION_FAILED
+	DB_EVENT_REP_INIT_DONE
+	DB_EVENT_REP_JOIN_FAILURE
+	DB_EVENT_REP_LOCAL_SITE_REMOVED
 	DB_EVENT_REP_MASTER
+	DB_EVENT_REP_MASTER_FAILURE
 	DB_EVENT_REP_NEWMASTER
 	DB_EVENT_REP_PERM_FAILED
+	DB_EVENT_REP_SITE_ADDED
+	DB_EVENT_REP_SITE_REMOVED
 	DB_EVENT_REP_STARTUPDONE
 	DB_EVENT_WRITE_FAILED
 	DB_EXCL
 	DB_EXTENT
 	DB_FAILCHK
+	DB_FAILCHK_ISALIVE
 	DB_FAST_STAT
 	DB_FCNTL_LOCKING
 	DB_FILEOPEN
@@ -164,6 +183,7 @@ BEGIN {
 	DB_FIXEDLEN
 	DB_FLUSH
 	DB_FORCE
+	DB_FORCESYNC
 	DB_FOREIGN_ABORT
 	DB_FOREIGN_CASCADE
 	DB_FOREIGN_CONFLICT
@@ -177,11 +197,19 @@ BEGIN {
 	DB_GET_BOTH_RANGE
 	DB_GET_RECNO
 	DB_GID_SIZE
+	DB_GROUP_CREATOR
 	DB_HANDLE_LOCK
 	DB_HASH
 	DB_HASHMAGIC
 	DB_HASHOLDVER
 	DB_HASHVERSION
+	DB_HEAP
+	DB_HEAPMAGIC
+	DB_HEAPOLDVER
+	DB_HEAPVERSION
+	DB_HEAP_FULL
+	DB_HEAP_RID_SZ
+	DB_HOTBACKUP_IN_PROGRESS
 	DB_IGNORE_LEASE
 	DB_IMMUTABLE_KEY
 	DB_INCOMPLETE
@@ -189,9 +217,11 @@ BEGIN {
 	DB_INIT_LOCK
 	DB_INIT_LOG
 	DB_INIT_MPOOL
+	DB_INIT_MUTEX
 	DB_INIT_REP
 	DB_INIT_TXN
 	DB_INORDER
+	DB_INTERNAL_DB
 	DB_JAVA_CALLBACK
 	DB_JOINENV
 	DB_JOIN_ITEM
@@ -201,10 +231,13 @@ BEGIN {
 	DB_KEYFIRST
 	DB_KEYLAST
 	DB_LAST
+	DB_LEGACY
+	DB_LOCAL_SITE
 	DB_LOCKDOWN
 	DB_LOCKMAGIC
 	DB_LOCKVERSION
 	DB_LOCK_ABORT
+	DB_LOCK_CHECK
 	DB_LOCK_CONFLICT
 	DB_LOCK_DEADLOCK
 	DB_LOCK_DEFAULT
@@ -260,13 +293,29 @@ BEGIN {
 	DB_LOG_LOCKED
 	DB_LOG_NOCOPY
 	DB_LOG_NOT_DURABLE
+	DB_LOG_NO_DATA
 	DB_LOG_PERM
 	DB_LOG_RESEND
 	DB_LOG_SILENT_ERR
+	DB_LOG_VERIFY_BAD
+	DB_LOG_VERIFY_CAF
+	DB_LOG_VERIFY_DBFILE
+	DB_LOG_VERIFY_ERR
+	DB_LOG_VERIFY_FORWARD
+	DB_LOG_VERIFY_INTERR
+	DB_LOG_VERIFY_PARTIAL
+	DB_LOG_VERIFY_VERBOSE
+	DB_LOG_VERIFY_WARNING
 	DB_LOG_WRNOSYNC
 	DB_LOG_ZERO
 	DB_MAX_PAGES
 	DB_MAX_RECORDS
+	DB_MEM_LOCK
+	DB_MEM_LOCKER
+	DB_MEM_LOCKOBJECT
+	DB_MEM_LOGID
+	DB_MEM_THREAD
+	DB_MEM_TRANSACTION
 	DB_MPOOL_CLEAN
 	DB_MPOOL_CREATE
 	DB_MPOOL_DIRTY
@@ -300,6 +349,8 @@ BEGIN {
 	DB_NEXT_NODUP
 	DB_NOCOPY
 	DB_NODUPDATA
+	DB_NOERROR
+	DB_NOFLUSH
 	DB_NOLOCKING
 	DB_NOMMAP
 	DB_NOORDERCHK
@@ -312,9 +363,11 @@ BEGIN {
 	DB_NOSYNC
 	DB_NOTFOUND
 	DB_NO_AUTO_COMMIT
+	DB_NO_CHECKPOINT
 	DB_ODDFILESIZE
 	DB_OK_BTREE
 	DB_OK_HASH
+	DB_OK_HEAP
 	DB_OK_QUEUE
 	DB_OK_RECNO
 	DB_OLD_VERSION
@@ -369,20 +422,25 @@ BEGIN {
 	DB_RENUMBER
 	DB_REPFLAGS_MASK
 	DB_REPMGR_ACKS_ALL
+	DB_REPMGR_ACKS_ALL_AVAILABLE
 	DB_REPMGR_ACKS_ALL_PEERS
 	DB_REPMGR_ACKS_NONE
 	DB_REPMGR_ACKS_ONE
 	DB_REPMGR_ACKS_ONE_PEER
 	DB_REPMGR_ACKS_QUORUM
 	DB_REPMGR_CONF_2SITE_STRICT
+	DB_REPMGR_CONF_ELECTIONS
 	DB_REPMGR_CONNECTED
 	DB_REPMGR_DISCONNECTED
+	DB_REPMGR_ISPEER
+	DB_REPMGR_NEED_RESPONSE
 	DB_REPMGR_PEER
 	DB_REP_ACK_TIMEOUT
 	DB_REP_ANYWHERE
 	DB_REP_BULKOVF
 	DB_REP_CHECKPOINT_DELAY
 	DB_REP_CLIENT
+	DB_REP_CONF_AUTOINIT
 	DB_REP_CONF_BULK
 	DB_REP_CONF_DELAYCLIENT
 	DB_REP_CONF_INMEM
@@ -453,6 +511,7 @@ BEGIN {
 	DB_SNAPSHOT
 	DB_SPARE_FLAG
 	DB_STAT_ALL
+	DB_STAT_ALLOC
 	DB_STAT_CLEAR
 	DB_STAT_LOCK_CONF
 	DB_STAT_LOCK_LOCKERS
@@ -462,6 +521,7 @@ BEGIN {
 	DB_STAT_MEMP_NOERROR
 	DB_STAT_NOERROR
 	DB_STAT_SUBSYSTEM
+	DB_STAT_SUMMARY
 	DB_ST_DUPOK
 	DB_ST_DUPSET
 	DB_ST_DUPSORT
@@ -501,7 +561,9 @@ BEGIN {
 	DB_TXN_ABORT
 	DB_TXN_APPLY
 	DB_TXN_BACKWARD_ROLL
+	DB_TXN_BULK
 	DB_TXN_CKP
+	DB_TXN_FAMILY
 	DB_TXN_FORWARD_ROLL
 	DB_TXN_LOCK
 	DB_TXN_LOCK_2PL
@@ -512,6 +574,7 @@ BEGIN {
 	DB_TXN_LOG_REDO
 	DB_TXN_LOG_UNDO
 	DB_TXN_LOG_UNDOREDO
+	DB_TXN_LOG_VERIFY
 	DB_TXN_NOSYNC
 	DB_TXN_NOT_DURABLE
 	DB_TXN_NOWAIT
@@ -521,6 +584,7 @@ BEGIN {
 	DB_TXN_REDO
 	DB_TXN_SNAPSHOT
 	DB_TXN_SYNC
+	DB_TXN_TOKEN_SIZE
 	DB_TXN_UNDO
 	DB_TXN_WAIT
 	DB_TXN_WRITE_NOSYNC
@@ -546,16 +610,20 @@ BEGIN {
 	DB_VERB_REP_MISC
 	DB_VERB_REP_MSGS
 	DB_VERB_REP_SYNC
+	DB_VERB_REP_SYSTEM
 	DB_VERB_REP_TEST
 	DB_VERB_WAITSFOR
 	DB_VERIFY
 	DB_VERIFY_BAD
 	DB_VERIFY_FATAL
 	DB_VERIFY_PARTITION
+	DB_VERSION_FAMILY
+	DB_VERSION_FULL_STRING
 	DB_VERSION_MAJOR
 	DB_VERSION_MINOR
 	DB_VERSION_MISMATCH
 	DB_VERSION_PATCH
+	DB_VERSION_RELEASE
 	DB_VERSION_STRING
 	DB_VRFY_FLAGMASK
 	DB_WRITECURSOR
@@ -567,6 +635,20 @@ BEGIN {
 	DB_YIELDCPU
 	DB_debug_FLAG
 	DB_user_BEGIN
+	LOGREC_ARG
+	LOGREC_DATA
+	LOGREC_DB
+	LOGREC_DBOP
+	LOGREC_DBT
+	LOGREC_Done
+	LOGREC_HDR
+	LOGREC_LOCKS
+	LOGREC_OP
+	LOGREC_PGDBT
+	LOGREC_PGDDBT
+	LOGREC_PGLIST
+	LOGREC_POINTER
+	LOGREC_TIME
 	);
 
 sub AUTOLOAD {
@@ -1047,6 +1129,78 @@ sub new
 
 *BerkeleyDB::Btree::TIEHASH = \&BerkeleyDB::Btree::new ;
 
+package BerkeleyDB::Heap ;
+
+use vars qw(@ISA) ;
+@ISA = qw( BerkeleyDB::Common BerkeleyDB::_tiedHash ) ;
+use UNIVERSAL ;
+use Carp ;
+
+sub new
+{
+    my $self = shift ;
+    my $got = BerkeleyDB::ParseParameters(
+		      {
+			# Generic Stuff
+			Filename 	=> undef,
+			Subname		=> undef,
+			#Flags		=> BerkeleyDB::DB_CREATE(),
+			Flags		=> 0,
+			Property	=> 0,
+			Mode		=> 0666,
+			Cachesize 	=> 0,
+			Lorder 		=> 0,
+			Pagesize 	=> 0,
+			Env		=> undef,
+			Txn		=> undef,
+			Encrypt		=> undef,
+
+			# Heap specific
+			HeapSize	=> undef,
+			HeapSizeGb	=> undef,
+		      }, @_) ;
+
+    croak("Env not of type BerkeleyDB::Env")
+        if defined $got->{Env} and ! UNIVERSAL::isa($got->{Env},'BerkeleyDB::Env');
+
+    croak("Txn not of type BerkeleyDB::Txn")
+        if defined $got->{Txn} and ! UNIVERSAL::isa($got->{Txn},'BerkeleyDB::Txn');
+
+#    if (defined $got->{HeapSize} )
+#    {
+#
+#        croak("-HeapSize needs a reference to a 2-element array")
+#            if $got->{HeapSize} !~ /ARRAY/ ||
+#
+#        croak("-HeapSize needs a reference to a 2-element array")
+#            if $got->{HeapSize} !~ /ARRAY/ ||
+#               @{ $got->{set_bt_compress} } != 2;
+#
+#        $got->{"HeapSize"} =  $got->{HeapSize}[0] 
+#            if defined $got->{HeapSize}[0];
+#
+#        $got->{"HeapSize"} =  $got->{HeapSize}[1] 
+#            if defined $got->{HeapSize}[1];
+#    }
+
+    BerkeleyDB::parseEncrypt($got);
+
+    my ($addr) = _db_open_heap($self, $got);
+    my $obj ;
+    if ($addr) {
+        $obj = bless [$addr] , $self ;
+	push @{ $obj }, $got->{Env} if $got->{Env} ;
+        $obj->Txn($got->{Txn}) 
+            if $got->{Txn} ;
+    }
+    return $obj ;
+}
+
+sub TIEHASH
+{
+    die "Tied Hash interface not supported with BerkeleyDB::Heap\n" ;
+}
+
 
 package BerkeleyDB::Recno ;
 
